from setuptools import setup

setup(
    name="inflexion",
    version="0.4",
    description="generic code for morphological analysis",
    license="MIT",
    url="http://github.com/jtauber/inflexion",
    author="James Tauber",
    author_email="jtauber@jtauber.com",
    packages=["inflexion"],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Topic :: Text Processing",
        "Topic :: Text Processing :: Linguistic",
    ],
)
